;; PLT Scheme front end to the STICCS
;; the Scheme TIme Code Calculator: STICCS
;
;;                     LICENSE AGREEMENT
;
; Copyright (c) 2004
;	Aaron Wen-yao Hsu.  All rights reserved.
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions
; are met:
; 1. Redistributions of source code must retain the above copyright
;    notice, this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Neither the name of the author nor the names of its contributors
;    may be used to endorse or promote products derived from this software
;    without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
; SUCH DAMAGE.
;
(load "backend.scm")

;LOGIC PROCESS OF CALCULATIONS
;
;The process is simple, there is one list, which serves as the stack 
;of the calculator. When a number is input through either the input 
;field or through the buttons, that number is converted to TC format 
;and then stored in the stack, the rest of the values being pushed 
;up the list. When an operator is encountered, the stack is checked 
;for values. If there is not more than one value, then the operator 
;itself is added to the stack. If there are at least two tc's in the 
;stack, then the operator is applied to those tc's, and the tc's are 
;replaced by the returned value of the operation. If a number is input 
;to the stack and there is an operator there already, that operator is 
;applied to that number and the number before the operator, and the 
;result is entered into the stack, replacing both the preceeding value 
;and the operator. If an operator is handled and there are no values 
;in the stack, then nothing is done. Each time an operation is 
;performed, the arguments and operators are printed to the tape, and 
;the result is printed below that.

; Define a global stack
(define stack (list))

;; tc-display : list -> string
; Takes a timecode and formats it to standard time format
(define (tc-display tc)
  (if (list? tc)
      (string-append
       (if (not (null? tc))
           (string-append
            (if (< (car tc) 10)
                (string-append "0" (number->string (car tc)))
                (number->string (car tc)))
            (if (not (null? (cdr tc))) ":" "")
            (tc-display (cdr tc)))
           ""))
      (number->string tc)))

;; tc->num : list -> number
; takes timecode and converts it to a "mul/div" number
(define (tc->num tc)
  (string->number (string-append
                   (number->string (car tc))
                   (number->string (cadr tc))
                   (number->string (caddr tc))
                   (number->string 
                    (cadddr tc)))))


;; tc-calc : list list char -> list
; takes two timecodes and an operand and outputs a timecode
(define (tc-calc tc1 tc2 opr)
  (display tc1) (display tc2) (display opr) (newline)
  (case opr
    ((add) (tc-add tc1 tc2 (string->number (send fps-f get-value))
                   (if (= (send fps-df get-selection) 0) #t #f)))
    ((subtract) (tc-sub tc1 tc2 (string->number (send fps-f get-value))
                        (if (= (send fps-df get-selection) 0) #t #f)))
    ((multiply) (tc-mul tc1 (if (list? tc1) (tc->num tc2) tc2)
                        (string->number (send fps-f get-value))
                        (if (= (send fps-df get-selection) 0) #t #f)))
    ((divide) (tc-div tc1 tc2
                      (string->number (send fps-f get-value))
                      (if (= (send fps-df get-selection) 0) #t #f)))))

;; out-print : list char -> undefined
; takes a timecode and a char and updates the output field according 
; to the method prescribed by the char with data string
(define (out-print tc com)
  (case com
    ((total)
     (send out-m set-value (string-append
                            (send out-m get-value)
                            "----------\n"
                            (tc-display tc) "\n")))
    ((add subtract divide multiply)
     (let ((com (case com
                  ((add) #\+)
                  ((subtract) #\-)
                  ((divide) #\/)
                  ((multiply) #\*))))
       (send out-m set-value (string-append
                              (send out-m get-value)
                              (tc-display tc)
                              " " (string com) "\n"))))
    (else
     (send out-m set-value (string-append
                            (send out-m get-value)
                            " \n"
                            (tc-display tc) "\n")))))

;; if-update : number -> undefined
; takes a number and updates the input field accordingly.
(define (if-update num)
  (cond ((equal? (send input-f get-value) "00000000")
         (send input-f set-value "")
         (send input-f set-value (string-append
                                  (send input-f get-value)
                                  (number->string num))))
        ((< (string-length (send input-f get-value)) 8)
         (send input-f set-value (string-append
                                  (send input-f get-value)
                                  (number->string num))))))

;; tc-proc : list list char list -> list
; takes two timecodes and an operator, updates the output window 
; and returns the total of the operation
(define (tc-proc tc1 tc2 opr)
  (out-print tc1 null)
  (out-print tc2 opr)
  (let ((tot (tc-calc tc1 tc2 opr)))
    (out-print tot 'total)
    tot))

(define main-f%
  (class frame%
    (override on-subwindow-char)
    (define on-subwindow-char 
      (lambda (windw evnt)
        (case (send evnt get-key-code)
          ((numpad-enter)
           (case (if (null? stack) #\nul (car stack))
             ((add subtract divide multiply)
              (set! stack (cons (tc-proc (cadr stack) 
                                         (get-tc 
                                          (send input-f get-value))
                                         (car stack))
                                (cddr stack))))
             (else (if (not (equal? (send input-f get-value) 
                                    "00000000"))
                       (set! stack (cons (get-tc 
                                          (send input-f get-value))
                                         stack)))))
           (send input-f set-value "00000000"))
          ((numpad1) (if-update 1))
          ((numpad2) (if-update 2))
          ((numpad3) (if-update 3))
          ((numpad4) (if-update 4))
          ((numpad5) (if-update 5))
          ((numpad6) (if-update 6))
          ((numpad7) (if-update 7))
          ((numpad8) (if-update 8))
          ((numpad9) (if-update 9))
          ((numpad0) (if-update 0))
          ((add subtract divide multiply)
           (cond ((equal? (send input-f get-value) "00000000")
                  (cond ((= (length stack) 1)
                         (set! stack (cons (send evnt get-key-code)
                                           stack)))
                        ((> (length stack) 1)
                         (if (and (list? (car stack))
                                  (list? (cadr stack)))
                             (set! stack (cons
                                          (tc-proc (cadr stack)
                                                   (car stack)
                                                   (send evnt
                                                         get-key-code))
                                          (cddr stack)))))))
                 (else
                  (cond ((= (length stack) 0)
                         (set! stack 
                               (list
                                (send evnt get-key-code)
                                (get-tc (send input-f get-value)))))
                        ((list? (car stack))
                         (set! stack
                               (cons
                                (tc-proc (car stack)
                                         (get-tc (send input-f
                                                       get-value))
                                         (send evnt get-key-code))
                                (cdr stack))))
                        ((symbol? (car stack))
                         (set! stack
                               (list (send evnt get-key-code)
                                     (tc-proc 
                                      (cadr stack)
                                      (get-tc (send input-f get-value))
                                      (car stack))
                                     (cddr stack)))))))
           (send input-f set-value "00000000")))))
    (super-instantiate ())))

; DEFINE GUI
(define main-w (instantiate main-f% ("STICCS")
                 (stretchable-width #f) (stretchable-height #f)))
(define input-f (instantiate text-field% ("" main-w void)
                  (init-value "00000000") (enabled #f)))
(define hor-p (instantiate horizontal-pane% (main-w)
                (stretchable-width #f)))
(define left-p (instantiate vertical-pane% (hor-p)
                 (alignment '(left top))))
(define fps-p (instantiate horizontal-pane% (left-p)
                (stretchable-width #f)))
(define fps-f (instantiate text-field% ("fps:" fps-p void)
                (init-value "30")))
(define fps-df (instantiate radio-box% ("" 
                                        '("Drop" "No Drop") 
                                        fps-p 
                                        void)))

(define br1-p (instantiate horizontal-pane% (left-p)))
(define br2-p (instantiate horizontal-pane% (left-p)))
(define br3-p (instantiate horizontal-pane% (left-p)))
(define br4-p (instantiate horizontal-pane% (left-p)))
(define br5-p (instantiate horizontal-pane% (left-p)))
(define btce-b (instantiate button% ("E" br1-p 
                                         (lambda (w e)
                                           (send input-f set-value
                                                 "00000000")
                                           (send out-m set-value "")
                                           (set! stack null)))
                 (min-width 27) (min-height 27)))
(define btcd-b (instantiate button% ("D" br1-p
                                         (lambda (w e)
                                           (set! stack 
                                                 (cdr stack))))
                 (min-width 27) (min-height 27)))
(define btc-b (instantiate button% ("C" br1-p 
                                        (lambda (w e) 
                                          (send input-f set-value
                                                "00000000"))) 
                (min-width 27) (min-height 27)))
(define bt/-b (instantiate button% 
                ("/" br1-p 
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'divide)))))
                (min-width 27) (min-height 27)))
(define bt7-b (instantiate button% 
                ("7" br2-p 
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad7)))))
                (min-width 27) (min-height 27)))
(define bt8-b (instantiate button% 
                ("8" br2-p 
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad8)))))
                (min-width 27) (min-height 27)))
(define bt9-b (instantiate button% 
                ("9" br2-p 
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad9)))))
                (min-width 27) (min-height 27)))
(define bt*-b (instantiate button% 
                ("*" br2-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'multiply)))))
                (min-width 27) (min-height 27)))
(define bt4-b (instantiate button% 
                ("4" br3-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad4)))))
                (min-width 27) (min-height 27)))
(define bt5-b (instantiate button% 
                ("5" br3-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad5)))))
                (min-width 27) (min-height 27)))
(define bt6-b (instantiate button% 
                ("6" br3-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad6)))))
                (min-width 27) (min-height 27)))
(define bt--b (instantiate button% 
                ("-" br3-p 
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'subtract)))))
                (min-width 27) (min-height 27)))
(define bt1-b (instantiate button% 
                ("1" br4-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad1)))))
                (min-width 27) (min-height 27)))
(define bt2-b (instantiate button% 
                ("2" br4-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad2)))))
                (min-width 27) (min-height 27)))
(define bt3-b (instantiate button% 
                ("3" br4-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad3)))))
                (min-width 27) (min-height 27)))
(define bt+-b (instantiate button% 
                ("+" br4-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'add)))))
                (min-width 27) (min-height 27)))
(define bt00-b (instantiate button% 
                 ("00" br5-p
                       (lambda (w e) 
                         (send main-w on-subwindow-char bt/-b
                               (new key-event% (key-code 'numpad0)))
                         (send main-w on-subwindow-char bt/-b
                               (new key-event% (key-code 'numpad0)))))
                 (min-width 27) (min-height 27)))
(define bt0-b (instantiate button% 
                ("0" br5-p
                     (lambda (w e) 
                       (send main-w on-subwindow-char bt/-b
                             (new key-event% (key-code 'numpad0)))))
                (min-width 27) (min-height 27)))
(define bt.-b (instantiate button% ("." br5-p void) 
                (min-width 27) (min-height 27)))
(define bte-b (instantiate button% 
                ("En" br5-p
                      (lambda (w e) 
                        (send main-w on-subwindow-char bt/-b
                              (new key-event% (key-code 'numpad-enter)))))
                (min-width 27) (min-height 27)))
(define right-p (instantiate vertical-pane% (hor-p)
                  (alignment '(right top))))
(define out-m (instantiate text-field% ("" right-p void)
                (init-value "")
                (min-width 130)
                (min-height 125)
                (style '(multiple))))

; Show and run initial operations
(send main-w show #t)